@@ -5,8 +5,9 @@ use strict;
 use warnings 'all';
 use HTTP::Date qw( time2iso str2time time2str );
 use Carp qw( croak confess );
+use HTTP::Headers;
+#use Scalar::Util 'weaken';
 use Apache2::ASP::Mock::RequestRec;
-use Apache2::ASP::HTTPContext::SubContext;
 
 our $MAX_BUFFER_LENGTH = 1024 ** 2;
 
@@ -19,14 +20,18 @@ sub new
   my ($class, %args) = @_;
   
   delete($args{context});
+  # Just guessing:
   my $s = bless {
+#    %args,
     _status           => 200,
     _output_buffer    => [ ],
     _do_buffer        => 1,
     _buffer_length    => 0,
+    _did_send_headers => 0,
   }, $class;
-  $s->ContentType('text/html'); 
-  $s->Expires( 0 );
+  $s->ContentType('text/html');
+  
+  $s->Expires( $args{_expires} || 0 );
   return $s;
 }# end new()
 
@@ -46,7 +51,7 @@ sub ContentType
   if( @_ )
   {
     confess "Response.ContentType cannot be changed after headers have been sent"
-      if $s->context->{_did_send_headers};
+      if $s->{_did_send_headers};
     $s->context->content_type( shift );
   }
   else
@@ -64,10 +69,9 @@ sub Status
   if( @_ )
   {
     confess "Response.Status cannot be changed after headers have been sent"
-      if $s->context->{_did_send_headers};
+      if $s->{_did_send_headers};
     
     $s->{_status} = shift;
-    $s->context->r->status( $s->{_status} );
   }
   else
   {
@@ -103,6 +107,7 @@ sub ExpiresAbsolute
   {
     $s->DeleteHeader('expires');
     $s->{_expires_absolute} = $when;
+#    $s->AddHeader( expires => shift );
   }
   else
   {
@@ -124,13 +129,12 @@ sub Redirect
   my ($s, $url) = @_;
   
   confess "Response.Redirect cannot be called after headers have been sent"
-    if $s->context->{_did_send_headers};
+    if $s->{_did_send_headers};
   
   $s->Clear;
   $s->AddHeader( location => $url );
   $s->Status( 302 );
   $s->End;
-  return 302; # New behavior - used to return '1':
 }# end Redirect()
 
 
@@ -140,26 +144,70 @@ sub End
   my $s = shift;
   
   $s->Flush;
-  $s->context->set_prop( did_end => 1 );
+  # Cancel execution and force the server to stop processing this request.
+  my $sock = $s->context->connection->client_socket;
+  $sock->close();
+#  eval { $sock->close() };
+  $s->context->{did_end} = 1;
 }# end End()
 
 
 #==============================================================================
 sub Flush
 {
-  my $s = shift;
+  my ($s) = @_;
   
+  if( $s->context->{parent} )
+  {
+    if( $IS_TRAPINCLUDE )
+    {
+      # Do nothing:
+      # We are not flushing - we are doing a Response.TrapInclude(...)
+    }
+    else
+    {
+    no strict 'refs';
+    my $parent = $s->context->{parent};
+#    local ${"$Apache2::ASP::HTTPContext::ClassName\::instance"} = $Apache2::ASP::HTTPContext::ClassName->new( parent => $parent );
+    local ${"$Apache2::ASP::HTTPContext::ClassName\::instance"} = $s->context->{parent};
+#      local $Apache2::ASP::HTTPContext::instance = $s->context->{parent};
+      return $s->context->response->Flush;
+    }# end if()
+  }# end if()
+  return unless $s->IsClientConnected;
+  $s->_send_headers unless $s->context->did_send_headers;
+  
+  no warnings 'uninitialized';
+  $s->context->print( join '', @{delete($s->{_output_buffer})} );
   $s->context->rflush;
+  $s->{_output_buffer} = [ ];
+  $s->{_buffer_length} = 0;
 }# end Flush()
 
 
 #==============================================================================
+our $WRITES = 0;
 sub Write
 {
   my $s = shift;
-  return unless defined($_[0]);
-
-  $s->context->print( shift );
+  return if $s->context->{did_end};
+  
+  if( $s->context->{parent} && ! $IS_TRAPINCLUDE )
+  {
+    no strict 'refs';
+    my $parent = $s->context->{parent};
+#    local ${"$Apache2::ASP::HTTPContext::ClassName\::instance"} = $Apache2::ASP::HTTPContext::ClassName->new( parent => $parent );
+    local ${"$Apache2::ASP::HTTPContext::ClassName\::instance"} = $s->context->{parent};
+#    local $Apache2::ASP::HTTPContext::instance = $s->context->{parent};
+    $s->context->response->Write( @_ );
+  }
+  else
+  {
+    no warnings 'uninitialized';
+    $s->{_buffer_length} += length($_[0]);
+    push @{$s->{_output_buffer}}, shift;
+    $s->Flush if (! $s->{_do_buffer}) || $s->{_buffer_length} >= $MAX_BUFFER_LENGTH;
+  }# end if()
 }# end Write()
 
 
@@ -170,7 +218,8 @@ sub Include
   return if $s->context->{did_end};
   
   my $ctx = $s->context;
-  my $subcontext = Apache2::ASP::HTTPContext::SubContext->new( parent => $ctx );
+  no strict 'refs';
+  local ${"$Apache2::ASP::HTTPContext::ClassName\::instance"} = $Apache2::ASP::HTTPContext::ClassName->new( parent => $ctx );
   
   my $root = $s->context->config->web->www_root;
   $path =~ s@^\Q$root\E@@;
@@ -181,17 +230,12 @@ sub Include
   use Apache2::ASP::Mock::RequestRec;
   my $clone_r = Apache2::ASP::Mock::RequestRec->new( );
   $clone_r->uri( $path );
-  $subcontext->setup_request( $clone_r, $ctx->cgi );
-  my $res = $subcontext->execute( $args );
-  $ctx->print( $subcontext->{r}->{buffer} );
-  $subcontext->DESTROY;
-
+  $s->context->setup_request( $clone_r, $ctx->cgi );
+  my $res = $s->context->execute( $args );
   if( $res > 200 )
   {
     $s->Status( $res );
   }# end if()
-
-  undef( $subcontext );
 }# end Include()
 
 
@@ -201,10 +245,9 @@ sub TrapInclude
   my ($s, $path, $args) = @_;
   return if $s->context->{did_end};
   
-  use Apache2::ASP::HTTPContext::SubContext;
-  
   my $ctx = $s->context;
-  my $subcontext = Apache2::ASP::HTTPContext::SubContext->new( parent => $ctx );
+  no strict 'refs';
+  local ${"$Apache2::ASP::HTTPContext::ClassName\::instance"} = $Apache2::ASP::HTTPContext::ClassName->new( parent => $ctx );
   
   my $root = $s->context->config->web->www_root;
   $path =~ s@^\Q$root\E@@;
@@ -212,16 +255,16 @@ sub TrapInclude
   local $ENV{SCRIPT_FILENAME} = $ctx->server->MapPath( $path );
   local $ENV{SCRIPT_NAME} = $path;
   
-  use Apache2::ASP::Mock::RequestRec;
   my $clone_r = Apache2::ASP::Mock::RequestRec->new( );
   $clone_r->uri( $path );
-  $subcontext->setup_request( $clone_r, $ctx->cgi );
-  my $res = $subcontext->execute( $args );
-  my $result = $subcontext->{r}->{buffer};
-  $subcontext->DESTROY;
+  $s->context->setup_request( $clone_r, $ctx->cgi );
 
-  undef( $subcontext );
-  return $result;
+  $IS_TRAPINCLUDE = 1;
+  $s->context->execute( $args );
+  $s->context->response->Flush;
+  
+  $IS_TRAPINCLUDE = 0;
+  return $clone_r->{buffer};
 }# end TrapInclude()
 
 
@@ -237,15 +280,12 @@ sub AddCookie
 {
   my $s = shift;
   
-  my ($name, $val, $path, $expires) = @_;
-  die "Usage: Response.AddCookie(name, value [, path [, expires ]])"
+  my ($name, $val) = @_;
+  die "Usage: Response.AddCookie(name, value)"
     unless defined($name) && defined($val);
-  $path ||= '/';
-  $expires ||= time() + ( 60 * 30 );
-  my $expire_date ||= time2str( $expires );
   
-  my $cookie = join '=', map { $s->context->cgi->escape( $_ ) } ( $name => $val );
-  $s->context->headers_out->push_header( 'set-cookie' => "$cookie; path=$path; expires=$expire_date" );
+  my $cookie = join '=', map { $s->context->r->cgi->escape( $_ ) } ( $name => $val );
+  $s->context->headers_out->push_header( 'set-cookie' => $cookie );
 }# end AddCookie()
 
 
@@ -256,7 +296,7 @@ sub AddHeader
   
   return unless defined($name) && defined($val);
   
-  return $s->context->headers_out->{ $name } = $val;
+  $s->context->headers_out->push_header( $name => $val );
 }# end AddHeader()
 
 
@@ -286,10 +326,24 @@ sub Clear
 #==============================================================================
 sub IsClientConnected
 {
-  return ! shift->context->get_prop('did_end');
+  return ! $_[0]->context->connection->aborted;
 }# end IsClientConnected()
 
 
+#==============================================================================
+sub _send_headers
+{
+  my $s = shift;
+  
+  my ($status) = $s->{_status} =~ m/^(\d+)/;
+  
+  $s->context->r->status( $status );
+  $s->context->content_type('text/html') unless $s->context->content_type;
+  $s->context->headers_out->push_header( Expires => $s->{_expires_absolute} );
+  $s->context->send_headers;
+}# end _send_headers()
+
+
 #==============================================================================
 sub DESTROY
 {
